package com.caimei.controller.search;

import com.caimei.controller.user.LoginController;
import com.caimei.entity.*;
import com.caimei.mapper.order.OrderMapper;

import com.caimei.module.search.entity.vo.*;
import com.caimei.module.search.service.SearchService;
import com.caimei.utils.AppUtils;
import com.caimei.utils.CopyUtil;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * 搜索
 */
@Controller
@RequestMapping("/search")
public class SearchController {
    @Autowired
    private SearchService searchService;
    @Autowired
    private LoginController loginController;
    @Resource
    private OrderMapper orderMapper;
    @Value("${miniprogram.domain}")
    private String domain;

    /**
     * 首页搜索商品
     */
    @ResponseBody
    @RequestMapping("/product")
    public CmWxJsonModel searchProduct(Integer organizeID, String searchWord, String code, Integer index,
                                       Integer pageSize, HttpServletRequest request) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        if (searchWord == null || "".equals(searchWord)) { return model.error("请输入商品关键词"); }
        CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
        String openID = ("1".equals(jsonModel.getCode())) ? (String)jsonModel.getData() : null;
        //查询历史记录是否存在
        CmMallProductSearchHistoryVo productSearchHistory = searchService.findAllProductRecord(organizeID, searchWord, openID);
        if (productSearchHistory == null) {
            //保存并删除大于10条的历史记录
            searchService.saveProductRecord(organizeID, searchWord, openID);
            searchService.deleteProductRecord(organizeID, openID);
        } else {
            searchService.updateProductRecord(organizeID, searchWord, openID);
        }
        index = (index != null) ? index : 0;
        pageSize = (pageSize != null) ? pageSize : 10;
        PageHelper.startPage(index, pageSize);
        List<CmMallOrganizeProductsVo> list = searchService.searchProduct(organizeID, searchWord);
        for (CmMallOrganizeProductsVo products : list) {
            products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
            //是否启用阶梯价格
            if (products.getLadderPriceFlag().equals("1")) {
                CmMallProductLadderPriceVo ladderPrice = searchService.findLadderPrice(products.getId().longValue());
                products.setRetailPrice(ladderPrice.getBuyPrice());
            }
        }
        CmMallPage<CmMallOrganizeProductsVo> productsPage = new CmMallPage<>(list);
        return CmWxJsonModel.newInstance().success(productsPage);
    }

    /**
     * 首页搜索历史记录
     */
    @ResponseBody
    @RequestMapping("/record")
    public CmWxJsonModel productRecord(Integer organizeID, String code, HttpServletRequest request) {
        CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
        String openID = (String) jsonModel.getData();
        List<CmMallProductSearchHistoryVo> historyList = searchService.findProductRecord(organizeID, openID);
        return CmWxJsonModel.newInstance().success(historyList);
    }

    /**
     * 首页搜索删除历史记录
     */
    @ResponseBody
    @RequestMapping("/delete")
    public CmWxJsonModel deleteProductRecord(Integer organizeID, String code, HttpServletRequest request) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        try {
            CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
            String openID = (String) jsonModel.getData();
            searchService.deleteRecord(organizeID, openID);
        } catch (Exception e) {
            return model.error("删除失败");
        }
        return model.success();
    }

    /**
     * 订单列表搜索订单
     */
    @ResponseBody
    @RequestMapping("/order")
    public CmWxJsonModel searchOrder(Integer organizeID, String searchWord, String code, Integer index,
                                     Integer pageSize, Long userID, HttpServletRequest request) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        if (searchWord == null || "".equals(searchWord)) { return model.error("请输入商品关键词"); }
        CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
        String openID = ("1".equals(jsonModel.getCode())) ? (String)jsonModel.getData() : null;
        //查询历史记录是否存在
        CmMallProductSearchHistoryVo orderSearchHistory = searchService.findAllOrderRecord(organizeID, searchWord, openID);
        if (orderSearchHistory == null) {
            //保存并删除大于10条的历史记录
            searchService.saveOrderRecord(organizeID, searchWord, openID);
            searchService.deleteOrderRecord(organizeID, openID);
        } else {
            searchService.updateOrderRecord(organizeID, searchWord, openID);
        }
        index = (index != null) ? index : 0;
        pageSize = (pageSize != null) ? pageSize : 10;
        PageHelper.startPage(index, pageSize);
        List<CmMallOrderVo> orderList = searchService.searchOrder(organizeID, searchWord, userID);
        List<CmMallOrder> orderList2 = new ArrayList<>(orderList.size());
        for (int i=0; i<orderList.size(); i++){
            CmMallOrder order = new CmMallOrder();
            CopyUtil.copyProperties(order, orderList.get(i));
            //111,待付待收待发
            if ("11".equals(order.getStatus()) && "1".equals(order.getPayStatus())) {
                order.setStatus("111");
            }
            String[] ShopOrderIDs = {};
            if (order.getShopOrderIDs().contains(",")) {
                ShopOrderIDs = order.getShopOrderIDs().split(",");
            } else {
                ShopOrderIDs = new String[]{order.getShopOrderIDs()};
            }
            List<CmMallShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
            for (CmMallShopOrder shopOrder : shopOrderList) {
                List<CmMallOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
                for (CmMallOrderProduct orderProduct : orderProductList) {
                    orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
                }
                shopOrder.setCmOrderProducts(orderProductList);
                shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
            }
            order.setShopOrderList(shopOrderList);
            orderList2.add(order);
        }
        CmMallPage<CmMallOrder> orderPage = new CmMallPage<>(orderList2);
        return model.success(orderPage);
    }

    /**
     * 订单搜索历史记录
     */
    @ResponseBody
    @RequestMapping("/history")
    public CmWxJsonModel orderRecord(Integer organizeID, String code, HttpServletRequest request) {
        CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
        String openID = (String) jsonModel.getData();
        List<CmMallProductSearchHistoryVo> historyList = searchService.findOrderRecord(organizeID, openID);
        return CmWxJsonModel.newInstance().success(historyList);
    }

    /**
     * 订单搜索删除历史记录
     */
    @ResponseBody
    @RequestMapping("/deleteOrderRecord")
    public CmWxJsonModel deleteOrderRecord(Integer organizeID, String code, HttpServletRequest request) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        try {
            CmWxJsonModel jsonModel = loginController.login(code, request, organizeID);
            String openID = (String) jsonModel.getData();
            searchService.deleteSearchHistory(organizeID, openID);
        } catch (Exception e) {
            return model.error("删除失败");
        }
        return model.success();
    }
}
