package com.caimei.controller.user;

import com.caimei.entity.*;
import com.caimei.service.user.LoginService;
import com.caimei.service.user.PersonalService;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 个人中心
 */
@Controller
@RequestMapping("/personal")
public class PersonalController {
    @Autowired
    private PersonalService personalService;
    @Autowired
    private LoginService loginService;

    /**
     * 我的个人显示数据
     *
     * @param operationUser
     * @return
     */
    @RequestMapping("/myCentre")
    @ResponseBody
    public CmWxJsonModel myCentre(CmMallOperationUser operationUser) {
        CmWxJsonModel res = CmWxJsonModel.newInstance();
        CmMallClub club = personalService.myCentre(operationUser);
        return res.success("1", club);
    }

    /**
     * 账户余额明细
     *
     * @return
     */
    @ResponseBody
    @RequestMapping("/touchBalance")
    public Map<String, Object> touchBalance(int year, int month, Integer index, Integer pageSize, CmMallUserBalanceRecord balanceRecord) {
        Map<String, Object> map = personalService.touchBalance(index, pageSize, year, month, balanceRecord);
        return map;
    }

    /**
     * 地址管理:省市区
     */
    @ResponseBody
    @RequestMapping("/address")
    public List<CmMallProvince> address() {
        List<CmMallProvince> list = personalService.address();
        return list;
    }

    /**
     * 省
     */
    @ResponseBody
    @RequestMapping("/province")
    public List<CmMallProvince> getProvince() {
        List<CmMallProvince> cmMallProvinceList = personalService.getProvince();
        return cmMallProvinceList;
    }

    /**
     * 市
     */
    @ResponseBody
    @RequestMapping("/city")
    public List<CmMallCity> getCity(Long provinceid) {
        List<CmMallCity> cmMallCityList = personalService.getCity(provinceid);
        return cmMallCityList;
    }

    /**
     * 区
     */
    @ResponseBody
    @RequestMapping("/town")
    public List<CmMallTown> getTown(Integer cityid) {
        List<CmMallTown> cmMallTownList = personalService.getTown(cityid);
        return cmMallTownList;
    }

    /**
     * 查询用户所有地址
     *
     * @param userID
     * @return
     */
    @ResponseBody
    @RequestMapping("/findAddress")
    public CmMallPage<CmMallAddress> findAddress(Integer userID, Integer index, Integer pageSize) {
        if (index == null) index = 1;
        if (pageSize == null) pageSize = 10;
        PageHelper.startPage(index, pageSize);
        List<CmMallAddress> addressList = personalService.findAddress(userID);
        CmMallPage<CmMallAddress> page = new CmMallPage<>(addressList);
        return page;
    }

    /**
     * 保存或编辑地址
     *
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public CmWxJsonModel saveAddress(@RequestBody CmMallAddress address, String code, HttpServletRequest request) {
        CmWxJsonModel res = CmWxJsonModel.newInstance();
        if (address == null) return res.error("参数异常");
        //判断是否处于登录状态
        String openid = (String) request.getSession().getAttribute("openid");
        CmMallOperationUser operationUser = loginService.doLogin(openid, address.getUserOrganizeID());
        if (operationUser == null) return res.error("账户异常");
        try {
            personalService.saveAddress(address);
        } catch (Exception e) {
            return res.error("保存失败");
        }
        return res.success("保存成功", "");
    }

    /**
     * 删除地址
     */
    @ResponseBody
    @RequestMapping("/delete")
    public CmWxJsonModel deleteAddress(Integer addressID, String code, Integer userOrganizeID, HttpServletRequest request) {
        CmWxJsonModel res = CmWxJsonModel.newInstance();
        //判断是否处于登录状态
        String openid = (String) request.getSession().getAttribute("openid");
        CmMallOperationUser operationUser = loginService.doLogin(openid, userOrganizeID);
        if (operationUser == null) return res.error("账户异常");
        try {
            personalService.deleteAddress(addressID);
        } catch (Exception e) {
            return res.error("删除失败");
        }
        return res.success("删除成功", "");
    }

    /**
     * 发票信息
     */
    @ResponseBody
    @RequestMapping("/invoice")
    public CmWxJsonModel invoice(@RequestBody CmMallUserInvoiceInfo userInvoiceInfo) {
        if (userInvoiceInfo == null) return CmWxJsonModel.newInstance().error("参数异常");
        CmWxJsonModel jsonModel = personalService.invoice(userInvoiceInfo);
        return jsonModel;
    }
}
