package com.caimei.service.order.impl;

import com.alibaba.fastjson.JSONArray;
import com.caimei.entity.*;
import com.caimei.mapper.order.BpUserInfoMapper;
import com.caimei.mapper.order.CmMallShopOrderMapper;
import com.caimei.mapper.order.OrderMapper;
import com.caimei.mapper.order.OrderProductMapper;
import com.caimei.mapper.products.DetailsMapper;
import com.caimei.mapper.products.HomePageMapper;
import com.caimei.mapper.products.OrganizeProductsMapper;
import com.caimei.mapper.products.ShoppingMapper;
import com.caimei.mapper.user.CmMallAddressMapper;
import com.caimei.mapper.user.LoginMapper;
import com.caimei.mapper.user.PersonalMapper;
import com.caimei.mapper.user.CmMallUserMapper;
import com.caimei.entity.CmMallOrganizeProducts;
import com.caimei.entity.CmMallProduct;
import com.caimei.entity.CmMallShop;
import com.caimei.service.order.OrderService;
import com.caimei.utils.AppUtils;
import com.caimei.utils.MathUtil;
import com.caimei.utils.NoUtils;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class OrderServiceImpl implements OrderService {
    protected static final Logger logger = LoggerFactory.getLogger(OrderServiceImpl.class);

    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private PersonalMapper personalMapper;
    @Autowired
    private LoginMapper loginMapper;
    @Autowired
    private CmMallUserMapper cmMallUserMapper;
    @Autowired
    private OrganizeProductsMapper organizeProductsMapper;
    @Autowired
    private OrderProductMapper orderProductMapper;
    @Autowired
    private CmMallShopOrderMapper cmMallShopOrderMapper;
    @Autowired
    private HomePageMapper homePageMapper;
    @Autowired
    private BpUserInfoMapper bpUserInfoMapper;
    @Autowired
    private CmMallAddressMapper cmMallAddressMapper;
    @Autowired
    private ShoppingMapper shoppingMapper;
    @Autowired
    private DetailsMapper detailsMapper;

    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public Map<String, Object> confirmOrder(Integer userId, String classifyIDS, Integer organizeID, String productIDs, Integer count) {
        Map<String, Object> map = new HashMap();
        //商品信息
        List<String> list = new ArrayList();
        if (productIDs.contains(",")) {
            String[] split = productIDs.split(",");
            list = Arrays.asList(split);
        } else {
            list.add(productIDs);
        }
        List<CmMallShop> cmMallShopList = orderMapper.findShop(list);
        for (CmMallShop cmMallShop : cmMallShopList) {
            List<CmMallOrganizeProducts> productsList = new ArrayList();
            Double retailPrice = 0d;//单价
            Double totalPrice = 0d;//单个商品总售价
            if (count == null) {
                //购物车购买
                Map<String, Object> params = new HashMap();
                params.put("productIDs", list);
                params.put("shopID", cmMallShop.getShopID());
                params.put("userId", userId);
                productsList = orderMapper.findAllCommodity(params);
                for (CmMallOrganizeProducts products : productsList) {
                    products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                    if(StringUtils.equals("1",products.getLadderPriceFlag())){//阶梯价格
                        List<CmMallProductLadderPrice> ladderPriceList = detailsMapper.findLadderPrice(products.getId());
                        //更加商品购买数量获取商品对应阶梯价格
                        retailPrice = getLadderPrice(products.getProductCount(), ladderPriceList);
                        totalPrice += retailPrice * products.getProductCount();
                        products.setRetailPrice(retailPrice);//重置阶梯价格售价
                    }else{
                        totalPrice += products.getRetailPrice() * products.getProductCount();
                    }
                }
            } else {
                //立即购买
                CmMallOrganizeProducts products = orderMapper.findCommodity(productIDs, cmMallShop.getShopID());
                products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                products.setProductCount(count);
                if(StringUtils.equals("1",products.getLadderPriceFlag())){//阶梯价格
                    List<CmMallProductLadderPrice> ladderPriceList = detailsMapper.findLadderPrice(products.getId());
                    //更加商品购买数量获取商品对应阶梯价格
                    retailPrice = getLadderPrice(products.getProductCount(), ladderPriceList);
                    totalPrice = retailPrice * products.getProductCount();
                    products.setRetailPrice(retailPrice);//重置阶梯价格售价
                }else{
                    totalPrice = products.getRetailPrice() * products.getProductCount();
                }
                productsList.add(products);
            }
            cmMallShop.setLogo(AppUtils.getImageURL("shopLogo", cmMallShop.getLogo(), 0, domain));
            cmMallShop.setTotalPrice(totalPrice);
            cmMallShop.setProductsList(productsList);
        }
        //发票信息
        CmMallUserInvoiceInfo invoice = personalMapper.findInvoice(userId);
        //运费
        CmMallFreePostageCard cmMallFreePostageCard = orderMapper.queryFree(userId);
        //-1到付，0包邮,-2仪器到付其它包邮
        boolean pinkage = true;//判断是否全部包邮
        boolean collect = true;//判断是否全部到付
        if (classifyIDS.contains(",")) {
            for (String classifyID : classifyIDS.split(",")) {
                if (classifyID != null && !classifyID.equals("")) {
                    CmMallProductsClassify productsClassify = orderMapper.findClassifyID(classifyID);
                    if (productsClassify.getPostageToPay().equals("1")) {
                        pinkage = false;
                    } else {
                        collect = false;
                    }
                }
            }
        } else {
            CmMallProductsClassify productsClassify = orderMapper.findClassifyID(classifyIDS);
            if (productsClassify.getPostageToPay().equals("1")) {
                pinkage = false;
            } else {
                collect = false;
            }
        }
        String freight = "";
        if (pinkage && cmMallFreePostageCard != null) {
            freight = "0";
        } else if (collect || pinkage || cmMallFreePostageCard == null) {
            freight = "-1";
        } else {
            freight = "-2";
        }
        //客服热线号码
        String cellPhone = loginMapper.cellPhone(organizeID);
        //可用余额
        Double userMoney = personalMapper.ableUserMoney(userId);
        if (userMoney == null) userMoney = 0d;
        map.put("shopList", cmMallShopList);
        map.put("invoice", invoice);
        map.put("freight", freight);
        map.put("cellPhone", cellPhone);
        map.put("userMoney", userMoney);
        return map;
    }

    @Override
    public String shareOrder(Integer orderID) throws Exception {
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) throw new Exception("订单不存在");
        OrderShareCode orderShareCode = orderMapper.queryCode(orderID);
        //生成分享码
        String shareCode = RandomStringUtils.random(4, false, true);
        //过期时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, 1);
        if (orderShareCode != null && orderShareCode.getExpiredTime().compareTo(new Date()) == 1) {
            return orderShareCode.getShareCode();
        } else if (orderShareCode != null) {
            orderShareCode.setShareCode(shareCode);
            orderShareCode.setExpiredTime(calendar.getTime());
            orderShareCode.setDelFlag("0");
            orderMapper.updateCode(orderShareCode);
            orderMapper.deleteCodeRecord(orderID);
            return String.valueOf(shareCode);
        } else {
            OrderShareCode code = new OrderShareCode();
            code.setOrderID(orderID);
            code.setShareCode(shareCode);
            code.setExpiredTime(calendar.getTime());
            code.setAddTime(new Date());
            code.setDelFlag("0");
            orderMapper.insertCode(code);
            return String.valueOf(shareCode);
        }
    }

    @Override
    public CmWxJsonModel verifyShareCode(String shareCode, String openid, Integer orderID, Integer userID,
                                         Integer organizeID) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) return model.error("-2", "该订单已删除");
        CmMallOperationUser operationUser = loginMapper.doLogin(openid, organizeID);
        if (operationUser != null && operationUser.getUserID().equals(userID)) {
            return model.success(true);
        }
        CmMallShareCodeRecord shareCodeRecord = orderMapper.findShareCodeRecord(openid, orderID);
        if (shareCodeRecord != null && shareCodeRecord.getExpiredTime().compareTo(new Date()) == 1) {
            return model.success();
        }
        OrderShareCode orderShareCode = orderMapper.queryCode(orderID);
        if (orderShareCode == null) return model.error("该订单没有生成分享码");
        if (orderShareCode.getShareCode().equals(shareCode) && orderShareCode.getExpiredTime().compareTo(new Date()) == 1) {
            CmMallShareCodeRecord codeRecord = new CmMallShareCodeRecord();
            codeRecord.setOpenid(openid);
            codeRecord.setAddTime(new Date());
            codeRecord.setOrderID(orderShareCode.getOrderID());
            codeRecord.setDelFlag("0");
            codeRecord.setShareCodeId(orderShareCode.getId());
            orderMapper.saveShareCodeRecord(codeRecord);
            return model.success();
        } else if (orderShareCode.getShareCode().equals(shareCode) && orderShareCode.getExpiredTime().compareTo(new Date()) != 1) {
            return model.error("分享码已失效");
        } else {
            return model.error("分享码错误");
        }
    }

    @Override
    public Map<String, Object> particulars(Integer orderID) throws Exception {
        Map<String, Object> map = new HashMap();
        //收货地址
        CmMallBpOrderUserInfo userInfo = orderMapper.findUserInfo(orderID);
        //订单信息,运费
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) throw new Exception("该订单已删除");
        order.setOrderMark("#" + order.getOrderID() + "#");
        //111,待付待收待发
        if ("11".equals(order.getStatus()) && "1".equals(order.getPayStatus())) {
            order.setStatus("111");
        }
        //判断交易全退情况下,是否发过货,77,交易全退可以查看物流
        List<CmMallLogisticsBatch> batchList = orderMapper.findLogistics(orderID);
        if ("7".equals(order.getStatus()) && batchList != null && batchList.size() > 0) {
            order.setStatus("77");
        }
        //商品信息
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        List<CmMallShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
        for (CmMallShopOrder shopOrder : shopOrderList) {
            List<CmMallOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
            for (CmMallOrderProduct orderProduct : orderProductList) {
                orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
            }
            shopOrder.setCmOrderProducts(orderProductList);
            shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
        }
        //发票信息
        CmMallOrderInvoice cmMallOrderInvoice = orderMapper.getOrderInvoice(orderID);
        //支付记录
        List<CmMallDiscernReceipt> discernReceiptList = orderMapper.findDiscernReceipt(orderID);
        if (discernReceiptList != null && discernReceiptList.size() > 0) {
            Double receiptAmount = 0d;
            for (CmMallDiscernReceipt discernReceipt : discernReceiptList) {
                receiptAmount += discernReceipt.getAssociateAmount();
            }
            //支付总金额
            order.setReceiptAmount(BigDecimal.valueOf(receiptAmount));
        }
        //退款记录
        List<CmMallReturnedPurchase> returnedPurchaseList = orderMapper.findReturnedPurchase(order.getOrderID().intValue());
        if (returnedPurchaseList != null && returnedPurchaseList.size() > 0) {
            Double returnedPurchaseFee = 0d;
            for (CmMallReturnedPurchase returnedPurchase : returnedPurchaseList) {
                returnedPurchaseFee += returnedPurchase.getRefundFee();
            }
            //退款总金额
            order.setReturnedPurchaseFee(BigDecimal.valueOf(returnedPurchaseFee));
        }
        //客服热线号码
        String cellPhone = loginMapper.cellPhone(order.getOrganizeID());
        //分享码
        String shareCode = orderMapper.findShareCode(orderID);
        map.put("order", order);
        map.put("userInfo", userInfo);
        map.put("shopOrderList", shopOrderList);
        map.put("orderInvoice", cmMallOrderInvoice);
        map.put("discernReceiptList", discernReceiptList);
        map.put("returnedPurchaseList", returnedPurchaseList);
        map.put("cellPhone", cellPhone);
        map.put("shareCode", shareCode);
        return map;
    }

    /**
     * 获取阶梯价格
     *
     * @return
     */
    public Double getLadderPrice(Integer number, List<CmMallProductLadderPrice> productLadderPrice) {
        Double ladderPrice = 0d;

        //已根据购买期数排序，期数越大价格越低
        Integer buyNum1 = 0;//第一阶梯（价格最高）
        Integer buyNum2 = 0;//第二阶梯
        Integer buyNum3 = 0;//第三阶梯（价格最低）
        if (productLadderPrice.size() == 3) {
            if (null != productLadderPrice.get(0)) buyNum1 = productLadderPrice.get(0).getBuyNum();
            if (null != productLadderPrice.get(1)) buyNum2 = productLadderPrice.get(1).getBuyNum();
            if (null != productLadderPrice.get(2)) buyNum3 = productLadderPrice.get(2).getBuyNum();
        } else if (productLadderPrice.size() == 2) {
            if (null != productLadderPrice.get(0)) buyNum1 = productLadderPrice.get(0).getBuyNum();
            if (null != productLadderPrice.get(1)) buyNum2 = productLadderPrice.get(1).getBuyNum();
        } else if (productLadderPrice.size() == 1) {
            if (null != productLadderPrice.get(0)) buyNum1 = productLadderPrice.get(0).getBuyNum();
        }
        if (null == buyNum1) buyNum1 = 0;
        if (null == buyNum2) buyNum2 = 0;
        if (null == buyNum3) buyNum3 = 0;

        //判断购买数量在什么阶段内
        if (buyNum3 != 0 && number >= buyNum3) {//3阶梯
            Double buyPrice = productLadderPrice.get(2).getBuyPrice();
            if (null == buyPrice) ladderPrice = 0d;
            ladderPrice = buyPrice;//购买的阶梯价格
        }
        if ((buyNum3 != 0 && buyNum2 != 0 && (number >= buyNum2 && number < buyNum3)) || (buyNum3 == 0 && buyNum2 != 0 && (number >= buyNum2))) {//2-3阶梯(只存在三个阶梯数据和值存在两个阶梯数据)
            if (productLadderPrice.size() == 1) {//只有一阶梯（一般不存在因为2阶梯已经不为0了）
                Double buyPrice = productLadderPrice.get(0).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;
            } else if (productLadderPrice.size() == 2) {//共有二阶梯
                Double buyPrice = productLadderPrice.get(1).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;
            } else if (productLadderPrice.size() == 3) {//共有三阶梯
                Double buyPrice = productLadderPrice.get(1).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;//购买的阶梯价格
            }
        }
        if ((buyNum2 != 0 && buyNum1 != 0 && (number >= buyNum1 && number < buyNum2)) || (buyNum2 == 0 && buyNum1 != 0 && (number >= buyNum1))) {//1-2阶梯（存在两个及以上阶梯数和只存在一个阶梯数据：两种情况）
            if (productLadderPrice.size() == 1) {//只有一阶梯
                Double buyPrice = productLadderPrice.get(0).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;
            } else if (productLadderPrice.size() == 2) {//共有二阶梯
                Double buyPrice = productLadderPrice.get(0).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;
            } else if (productLadderPrice.size() == 3) {//共有三阶梯
                Double buyPrice = productLadderPrice.get(0).getBuyPrice();
                if (null == buyPrice) buyPrice = 0d;
                ladderPrice = buyPrice;
            }
        }

        if ((buyNum1 != 0 && (number <= buyNum1))) {//1阶梯
            Double buyPrice = productLadderPrice.get(0).getBuyPrice();
            if (null == buyPrice) buyPrice = 0d;
            ladderPrice = buyPrice;
        }
        return ladderPrice;
    }


    @Override
    @Transactional
    public CmWxJsonModel saveOrderInfo(CmWxJsonModel cmWxJsonModel, Integer userId, Integer organizeID, String cartType, Integer addressID, Object orderInfo, String balanceDeductionFlag, Double orderShouldPayFee, Integer cmOperationID) {
        /**获取用户信息*/
        CmMallUser cmMallUser = cmMallUserMapper.findUserById(userId);
        /**获取用户包邮卡*/
        CmMallFreePostageCard cmMallFreePostageCard = orderMapper.queryFree(userId);
        if (null == cmMallUser) {
            return cmWxJsonModel.error("-1", "用户账户异常");
        }
        Integer clubID = cmMallUser.getClubID();
        boolean isIncludedInstruments = false;//是否包含仪器(默认不包含仪器)
        boolean isIncludedProduct = false;//是否包含商品(默认不包含仪器)
        boolean isPaySuccessFlag = false;//是完成支付(默认不是，只有余额抵扣才算)
        CmMallOrder cmMallOrder = new CmMallOrder();
        List<CmMallOrderProduct> orderProductList = new ArrayList<>();//整理订单商品列表
        HashMap<Integer, String> shopIDs = new HashMap<>();
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateStr = sdf.format(date);

        /**订单信息转化*/
        List<Map<String, Object>> orderInfoList = (List<Map<String, Object>>) orderInfo;
        if (null != orderInfoList) {
            Integer productCount = 0;//统计购买所有商品的数量
            double productTotalFee = 0d;//商品总金额 （商品单价乘以数量，再加上税费[默认0]）
            //double orderTotalFee = 0d;//小计金额 （商品折后单价乘以数量，再加上税费[默认0]）商城下单和商品总金额一样
            double payTotalFee = 0d;//订单总额（小计金额减去经理折扣后，再加上运费[默认0])
            BigDecimal payableAmount = new BigDecimal(0);//真实支付金额（订单总额减去抵扣的账户余额）
            double balancePayFee = 0d;//余额支付金额
            for (Map<String, Object> shopOrderInfo : orderInfoList) {//遍历订单商品信息
                Integer shopId = (Integer) shopOrderInfo.get("shopId");
                String note = (String) shopOrderInfo.get("note");
                Object productInfo = shopOrderInfo.get("productInfo");//一个子订单对应的商品信息
                if (null == shopId) return cmWxJsonModel.error("-1", "供应商数据异常");
                if (null == productInfo) return cmWxJsonModel.error("-1", "订单商品数据异常");
                shopIDs.put(shopId, note);

                /**获取子订单商品信息*/
                List<Map<String, Object>> productInfoList = (List<Map<String, Object>>) productInfo;
                if (!CollectionUtils.isEmpty(productInfoList)) {
                    for (Map<String, Object> product : productInfoList) {//遍历所有商品
                        Integer productId = (Integer) product.get("productId");
                        Integer productNum = (Integer) product.get("productNum");
                        //统计商品总数量
                        productCount += productNum;
                        //获取商品信息
                        CmMallOrganizeProducts cmMallOrganizeProducts = organizeProductsMapper.selectProductById(productId);
                        if (null == cmMallOrganizeProducts) return cmWxJsonModel.error("-1", "订单商品数据异常");
                        if (null == productNum || productNum == 0) return cmWxJsonModel.error("-1", "商品购买数量异常");
                        CmMallProduct cmCmMallProduct = organizeProductsMapper.selectCmProductById(cmMallOrganizeProducts.getProductID());
                        Double retailPrice = 0d;//售价
//                        Integer classifyID = cmMallOrganizeProducts.getClassifyID();

                        if(StringUtils.equals("1",cmMallOrganizeProducts.getLadderPriceFlag())){
                            List<CmMallProductLadderPrice> ladderPriceList = detailsMapper.findLadderPrice(productId);
                            //判断阶梯价格的购买数量校验
                            Integer minBuyNumber = getMinBuyNumber(ladderPriceList);
                            if(productNum < minBuyNumber){
                                return cmWxJsonModel.error("-1", "商品购买量低于最小起订量");
                            }
                            //更加商品购买数量获取商品对应阶梯价格
                            retailPrice = getLadderPrice(productNum, ladderPriceList);
                        }else{
                            retailPrice = cmMallOrganizeProducts.getRetailPrice();
                        }
                        double productFee = retailPrice * productNum;//单个商品的金额
                        //统计商品总金额
                        productTotalFee += productFee;
                        if (StringUtils.equals("1", cmMallOrganizeProducts.getPostageToPay()) && !isIncludedInstruments) {//判断是否存在仪器
                            isIncludedInstruments = true;
                        }
                        if (!StringUtils.equals("1", cmMallOrganizeProducts.getPostageToPay()) && !isIncludedProduct) {//判断是否存在商品
                            isIncludedProduct = true;
                        }

                        /**整理订单商品集合数据*/
                        CmMallOrderProduct cmMallOrderProduct = new CmMallOrderProduct();
                        cmMallOrderProduct.setShopID(cmMallOrganizeProducts.getShopID());
                        cmMallOrderProduct.setProductID(cmMallOrganizeProducts.getProductID());//此处保存采美库商品Id
                        cmMallOrderProduct.setOrganizeProductID(cmMallOrganizeProducts.getId());
                        cmMallOrderProduct.setOrganizeID(organizeID);
                        //预留在保存保存子订单的时候添加
                        cmMallOrderProduct.setProductNo(null);
                        cmMallOrderProduct.setNum(productNum);
                        cmMallOrderProduct.setPresentNum(0);
                        cmMallOrderProduct.setPrice(retailPrice);
                        cmMallOrderProduct.setProductUnit(cmCmMallProduct.getUnit());
                        cmMallOrderProduct.setNormalPrice(cmMallOrganizeProducts.getNormalPrice());
                        cmMallOrderProduct.setCostPrice(cmMallOrganizeProducts.getCostPrice());
                        cmMallOrderProduct.setShopProductAmount(cmMallOrganizeProducts.getCostPrice() * productNum);//商品费=成本价快照*(购买数量  + 赠品数量)
                        cmMallOrderProduct.setPrice0(retailPrice);
                        cmMallOrderProduct.setPrice1(retailPrice);
                        cmMallOrderProduct.setTaxRate(0d);
                        cmMallOrderProduct.setAddedValueTax(0d);
                        cmMallOrderProduct.setTotalAddedValueTax(0d);
                        cmMallOrderProduct.setShopFee(cmMallOrganizeProducts.getCostPrice() * productNum);//应付供应商=成本价 * 商品数量
                        cmMallOrderProduct.setSingleShopFee(cmMallOrganizeProducts.getCostPrice());
                        cmMallOrderProduct.setOtherFee(0d);
                        cmMallOrderProduct.setSingleOtherFee(0d);
                        cmMallOrderProduct.setCmFee((retailPrice * productNum) - (cmMallOrganizeProducts.getCostPrice() * productNum));//应付采美=应付金额(应收金额)-应付供应商-应付第三方0-税费0）
                        cmMallOrderProduct.setSingleCmFee(retailPrice - cmMallOrganizeProducts.getCostPrice());
                        cmMallOrderProduct.setTotalAmount(retailPrice * productNum);
                        cmMallOrderProduct.setTotalFee(retailPrice * productNum);
                        cmMallOrderProduct.setShouldPayFee(retailPrice * productNum);
                        cmMallOrderProduct.setTotalBeans(0d);
                        cmMallOrderProduct.setUseBalanceAmount(0d);
                        cmMallOrderProduct.setDiscountFee(0D);//经理折扣 优惠金额
                        cmMallOrderProduct.setPreferential(0d);//优惠金额
                        cmMallOrderProduct.setUseBalanceAmount(0d);
                        cmMallOrderProduct.setConfirmProductFlag("0");//订单商品供应商确认标志 0否 1是
                        cmMallOrderProduct.setDiscount(100d);
                        cmMallOrderProduct.setDiscountPrice(retailPrice);
                        cmMallOrderProduct.setShopName(cmMallOrganizeProducts.getShopName());
                        cmMallOrderProduct.setName(cmCmMallProduct.getName());
                        cmMallOrderProduct.setPayStatus("0");
                        cmMallOrderProduct.setBuyAgainFlag("0");
                        cmMallOrderProduct.setNotOutStore(productNum); // 未出库数量
                        cmMallOrderProduct.setCommentFlag("0");//是否已评论 1 是 0 未评论
                        cmMallOrderProduct.setActPreferential(0d);
                        cmMallOrderProduct.setActType(null);
                        cmMallOrderProduct.setIsActProduct("0");//非活动商品
                        cmMallOrderProduct.setProductImage(AppUtils.getImageURL("product", cmCmMallProduct.getMainImage(), 0, domain));
                        orderProductList.add(cmMallOrderProduct);
                    }
                }
            }

            /**判断免邮标志*/
            if (isIncludedInstruments) {//包括仪器
                if (isIncludedProduct && null != cmMallFreePostageCard) {//包含商品且有包邮卡
                    cmMallOrder.setFreePostFlag("-2");//免邮标志：-1到付，0包邮，1需要运费,-2仪器到付其它包邮【小程序只存在-1,0,-2】
                    cmMallOrder.setFreight(BigDecimal.valueOf(-2));
                } else {//（1包含商品无包邮卡，2不包含商品）
                    cmMallOrder.setFreePostFlag("-1");
                    cmMallOrder.setFreight(BigDecimal.valueOf(-1));
                }
            } else {//不包括仪器
                if (isIncludedProduct && null != cmMallFreePostageCard) {//无仪器包含商品且有包邮卡
                    cmMallOrder.setFreePostFlag("0");//免邮标志：-1到付，0包邮，1需要运费,-2仪器到付其它包邮【小程序只存在-1,0,-2】
                    cmMallOrder.setFreight(BigDecimal.valueOf(0));
                } else {//（1无仪器包含商品无包邮卡）
                    cmMallOrder.setFreePostFlag("-1");
                    cmMallOrder.setFreight(BigDecimal.valueOf(-1));
                }
            }

            /**统计订单应付总金额*/
            payTotalFee = productTotalFee;
            BigDecimal bdPayTotalFee = new BigDecimal(payTotalFee);
            cmMallOrder.setStatus("11");//待收款待发货(设置初始化状态)
            cmMallOrder.setReceiptStatus("1");//（收款买家）收款状态：1待收款、2部分收款、3已收款(设置初始化状态)
            cmMallOrder.setPayFlag("0");//是否已支付 未支付0 已支付1(设置初始化状态)

            /**判断余额使用*/
            boolean useBalanceFlag = false;//默认不适用余额
            if (StringUtils.equals("1", balanceDeductionFlag)) {//勾选余额抵扣
                //获取用户最新余额
                Double userMoney = personalMapper.ableUserMoney(userId);
                logger.info(">>>>>" + userId + "userMoney:" + userMoney);
                if (null != userMoney && userMoney > 0d) {
                    useBalanceFlag = true;
                    BigDecimal bdUserMoney = new BigDecimal(userMoney);
                    if (payTotalFee > userMoney) {//部分抵扣
                        balancePayFee = userMoney;
                        payableAmount = bdPayTotalFee.subtract(bdUserMoney);
                        cmMallUser.setUserMoney(new BigDecimal(0));//余额抵扣用完
                        cmMallUser.setAbleUserMoney(new BigDecimal(0));//余额抵扣用完
                        cmMallUserMapper.updateUserMoney(cmMallUser);
                        cmMallUserMapper.updateAbleUserMoney(cmMallUser);
                        cmMallOrder.setStatus("21");//部收款待发货
                        cmMallOrder.setReceiptStatus("2");
                    } else if (payTotalFee < userMoney) {//全部用余额抵扣，无需直接，直接变成支付完成
                        balancePayFee = payTotalFee;
                        cmMallUser.setAbleUserMoney(bdUserMoney.subtract(bdPayTotalFee));
                        cmMallUser.setUserMoney(bdUserMoney.subtract(bdPayTotalFee));
                        cmMallUserMapper.updateAbleUserMoney(cmMallUser);
                        cmMallUserMapper.updateUserMoney(cmMallUser);
                        cmMallOrder.setStatus("31");//已收款待发货
                        cmMallOrder.setReceiptStatus("3");
                        cmMallOrder.setPayFlag("1");
                        isPaySuccessFlag = true;
                    } else if (payTotalFee == userMoney) {//刚好抵扣完
                        balancePayFee = payTotalFee;
                        cmMallUser.setAbleUserMoney(new BigDecimal(0));
                        cmMallUser.setUserMoney(new BigDecimal(0));
                        cmMallUserMapper.updateAbleUserMoney(cmMallUser);
                        cmMallUserMapper.updateUserMoney(cmMallUser);
                        cmMallOrder.setStatus("31");//已收款待发货
                        cmMallOrder.setReceiptStatus("3");
                        cmMallOrder.setPayFlag("1");
                        isPaySuccessFlag = true;
                    }
                    cmMallOrder.setPayTime(dateStr);//支付时间

                }
            } else {
                payableAmount = new BigDecimal(payTotalFee);//无余额支付的时就是真实支付金额
            }

            /**判断前端传入orderShouldPayFee订单应付金额,和后台计算应付金额对比*/
            double v = payableAmount.subtract(new BigDecimal(orderShouldPayFee)).doubleValue();
            logger.info(">>>>>payableAmount:" + payableAmount + "orderShouldPayFee:" + orderShouldPayFee);
            if (v < -0.1d || v > 0.1d) {//考虑前端计算不精确
                return cmWxJsonModel.error("-1", "订单付款金额异常");
            }

            /**判断发票使用*/
            CmMallUserInvoiceInfo info = personalMapper.findInvoice(userId);
            if (null != info) {
                cmMallOrder.setInvoiceFlag("2");//是否开发票 没开发票0 /开个人发票1 /开企业发票2【小程序目前只存在0,2】
            } else {
                cmMallOrder.setInvoiceFlag("0");
            }

            /**初始化主订单参数*/
            String orderNo = NoUtils.getCmOrderNo("X");
            cmMallOrder.setOrderNo(orderNo);
            cmMallOrder.setOrganizeID(organizeID);
            cmMallOrder.setUserID(userId.longValue());
            cmMallOrder.setBuyUserID(cmOperationID);//操作者的ID(下单人)
            cmMallOrder.setOrderSubmitType(2);// 2:员工帮会所下单
            cmMallOrder.setOrderType(1);//订单类型 协销订单0 、普通订单(自主订单)1
            cmMallOrder.setHasActProduct("0");
            cmMallOrder.setSendOutStatus("1");//待发货
            cmMallOrder.setProductTotalFee(BigDecimal.valueOf(productTotalFee));
            cmMallOrder.setOrderTotalFee(BigDecimal.valueOf(productTotalFee));
            cmMallOrder.setPayTotalFee(BigDecimal.valueOf(payTotalFee));
            cmMallOrder.setPayableAmount(payableAmount);
            cmMallOrder.setPreferential(BigDecimal.valueOf(0));
            cmMallOrder.setDiscountFee(BigDecimal.valueOf(0));//经理折扣
            cmMallOrder.setSpID(null);
            cmMallOrder.setMainSpID(null);
            cmMallOrder.setClubID(clubID.longValue());
            cmMallOrder.setOrderSource("6");//小程序
            cmMallOrder.setConfirmTime(dateStr);
            cmMallOrder.setOrderTime(dateStr);
            cmMallOrder.setProductCount(productCount);
            cmMallOrder.setPresentCount(0);
            cmMallOrder.setConfirmFlag("1");
            cmMallOrder.setClauseID(1l);
            cmMallOrder.setClauseContent(null);
            cmMallOrder.setClauseName(null);
            cmMallOrder.setUpdateDate(dateStr);
            cmMallOrder.setDelFlag("0");
            cmMallOrder.setSplitFlag("0");//0不可拆分
            cmMallOrder.setPostageOrderFlag("0");//0不是邮费订单
            cmMallOrder.setBalancePayFee(BigDecimal.valueOf(balancePayFee));//余额支付金额
            cmMallOrder.setPayStatus("1");//(付款供应商)付款状态：1待付款、2部分付款、3已付款
            cmMallOrder.setOnlinePayFlag("0");//是否能走线上支付 0可以 1不可以 只能线下

            //保存住订单数据
            orderMapper.insert(cmMallOrder);
            Long orderID = cmMallOrder.getOrderID();
            logger.info(">>>>>orderID：" + orderID);

            if (useBalanceFlag) {
                /**添加余额收支记录*/
                CmMallUserBalanceRecord cmMallUserBalanceRecord = new CmMallUserBalanceRecord();
                cmMallUserBalanceRecord.setUserId(userId);
                cmMallUserBalanceRecord.setType("2");
                cmMallUserBalanceRecord.setBalanceType("1");
                cmMallUserBalanceRecord.setAddDate(new Date());
                cmMallUserBalanceRecord.setAmount(String.valueOf(balancePayFee));
                cmMallUserBalanceRecord.setOrderId(cmMallOrder.getOrderID().intValue());
                cmMallUserBalanceRecord.setDelFlag("0");
                cmMallUserMapper.insertUserBalanceRecord(cmMallUserBalanceRecord);
            }

            /**保存订单商品*/
            for (CmMallOrderProduct product : orderProductList) {//循环插入数据
                product.setOrderID(orderID.intValue());
                product.setOrderNo(orderNo);
                //保存订单商品数据
                organizeProductsMapper.insert(product);
            }

            String shopOrderIDs = "";//收集子订单信息
            /**保存子订单*/
            Iterator<Map.Entry<Integer, String>> iterator = shopIDs.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<Integer, String> shopOrder = iterator.next();
                Integer shopID = shopOrder.getKey();
                String note = shopOrder.getValue();

                /**统计子订单金额信息*/
                CmMallOrderProduct cmMallOrderProduct = new CmMallOrderProduct();
                cmMallOrderProduct.setOrderID(cmMallOrder.getOrderID().intValue());
                cmMallOrderProduct.setShopID(shopID);
                //查询属于该供应商的所有订单商品信息
                List<CmMallOrderProduct> cmMallOrderProductList = orderProductMapper.findList(cmMallOrderProduct);
                BigDecimal totalAmount = new BigDecimal(0); // 订单总金额
                BigDecimal productAmount = new BigDecimal(0); //商品总金额
                BigDecimal needPayAmount = new BigDecimal(0);//需要支付金额
                BigDecimal preferential = new BigDecimal(0);// 优惠金额
                BigDecimal brokerage = new BigDecimal(0);//佣金
                Double shopProductAmount = 0D;//商品费
                Integer buyNum = 0;//总购买数
                //计算子订单信息
                for (CmMallOrderProduct orderProduct : cmMallOrderProductList) {
                    productAmount = MathUtil.add(productAmount, orderProduct.getTotalAmount());//商品总金额 包括赠品
                    totalAmount = MathUtil.add(totalAmount, orderProduct.getTotalFee());//订单总金额 包括税费
                    needPayAmount = MathUtil.add(needPayAmount, orderProduct.getShouldPayFee()); //应付金额
                    buyNum += orderProduct.getNum();//总购买数
                    preferential = MathUtil.add(preferential, orderProduct.getPreferential());
                    brokerage = MathUtil.add(brokerage, orderProduct.getCmFee());
                    shopProductAmount += (orderProduct.getShopProductAmount() == null ? 0D : orderProduct.getShopProductAmount());
                }

                //初始化子订单信息
                CmMallShopOrder cmMallShopOrder = new CmMallShopOrder();
                cmMallShopOrder.setOrganizeID(organizeID);
                cmMallShopOrder.setClubID(clubID);
                cmMallShopOrder.setSpID(null);
                cmMallShopOrder.setMainSpID(null);
                cmMallShopOrder.setOrderID(orderID.intValue());
                cmMallShopOrder.setOrderNo(orderNo);
                cmMallShopOrder.setUserID(userId);
                cmMallShopOrder.setNote(note);
                cmMallShopOrder.setOrderTime(dateStr);
                cmMallShopOrder.setDiscountFee(0d);
                cmMallShopOrder.setCanRefundFlag(1);
                cmMallShopOrder.setCanRefundAmount(needPayAmount.doubleValue());
                cmMallShopOrder.setAccountAmount(0d);
                cmMallShopOrder.setBrokerage(brokerage.doubleValue());//佣金 采美应收
                cmMallShopOrder.setBuyStatus("1");
                cmMallShopOrder.setPresentNum(0);
                cmMallShopOrder.setUseBeanFlag(0);
                cmMallShopOrder.setUseBeanAmount(0);
                cmMallShopOrder.setUseBalanceFlag(0);
                cmMallShopOrder.setRefundStatus(0);
                cmMallShopOrder.setRefundsAmount(0d);
                cmMallShopOrder.setPayStatus("1");
                cmMallShopOrder.setSendOutStatus("1");
                cmMallShopOrder.setPayFlag("0");
                cmMallShopOrder.setOrderStatusFlag("1");
                cmMallShopOrder.setDelFlag("0");
                cmMallShopOrder.setItemCount(buyNum);//购买商品数
                cmMallShopOrder.setPresentNum(0);//赠送数
                cmMallShopOrder.setShopID(shopID);
                cmMallShopOrder.setOrderType(1);
                cmMallShopOrder.setOrderSubmitType(cmMallOrder.getOrderSubmitType());
                cmMallShopOrder.setTotalAmount(totalAmount.doubleValue());
                cmMallShopOrder.setProductAmount(productAmount.doubleValue());
                cmMallShopOrder.setNeedPayAmount(needPayAmount.doubleValue());
                cmMallShopOrder.setPreferential(preferential.doubleValue());
                cmMallShopOrder.setShopProductAmount(shopProductAmount);
                cmMallShopOrder.setShopPostFee(0d);//付给供应商运费
                cmMallShopOrder.setShopTaxFee(0d);//付给供应商税费
                cmMallShopOrder.setPayedShopAmount(0d);//已付款金额
                cmMallShopOrder.setShopOtherFee(0d);//付第三方
                cmMallShopOrder.setSplitFlag("0");

                List<CmMallShopOrder> cmMallShopOrders1 = cmMallShopOrderMapper.findMaxShopOrderNo(cmMallOrder.getOrderID().intValue());
                String shopOrderNo = ""; //子订单编号
                if (!CollectionUtils.isEmpty(cmMallShopOrders1)) {
                    shopOrderNo = cmMallShopOrders1.get(0).getShopOrderNo();
                    cmMallShopOrder.setShopOrderNo(NoUtils.getCmShopOrderNo(cmMallOrder.getOrderNo(), Integer.parseInt(shopOrderNo.substring(shopOrderNo.length() - 2, shopOrderNo.length())) + 1));
                } else {
                    cmMallShopOrder.setShopOrderNo(NoUtils.getCmShopOrderNo(cmMallOrder.getOrderNo(), 1));
                }
                //保存子订单信息到数据库
                cmMallShopOrderMapper.insert(cmMallShopOrder);
                shopOrderIDs += ((shopOrderIDs.equals("") ? "" : ",") + cmMallShopOrder.getShopOrderID());
                for (CmMallOrderProduct orderProduct : cmMallOrderProductList) {
                    orderProduct.setShopOrderID(cmMallShopOrder.getShopOrderID());
                    orderProduct.setShopOrderNo(cmMallShopOrder.getShopOrderNo());
                    orderProductMapper.update(orderProduct);
                }
            }

            //更新主订单信息
            cmMallOrder.setShopOrderIDs(shopOrderIDs);//子订单ID:1000,1002
            orderMapper.update(cmMallOrder);

            //获取地址信息
            CmMallAddress cmMallAddress = cmMallAddressMapper.findByAddressID(addressID);
            if (null != cmMallAddress) {
                //保存地址信息
                CmMallBpOrderUserInfo cmMallBpOrderUserInfo = new CmMallBpOrderUserInfo();
                cmMallBpOrderUserInfo.setOrderId(orderID.intValue());
                cmMallBpOrderUserInfo.setClubId(clubID);
                cmMallBpOrderUserInfo.setUserId(userId);
                cmMallBpOrderUserInfo.setName(cmMallUser.getUserName());
                cmMallBpOrderUserInfo.setShouHuoRen(cmMallAddress.getShouHuoRen());
                cmMallBpOrderUserInfo.setMobile(cmMallAddress.getMobile());
                cmMallBpOrderUserInfo.setPostalCode(cmMallAddress.getPhone());
                cmMallBpOrderUserInfo.setPostalCode(cmMallAddress.getPostalCode());
                cmMallBpOrderUserInfo.setTownId(cmMallAddress.getTownID());
                cmMallBpOrderUserInfo.setProvince(cmMallAddress.getProvince());
                cmMallBpOrderUserInfo.setCity(cmMallAddress.getCity());
                cmMallBpOrderUserInfo.setTown(cmMallAddress.getTown());
                cmMallBpOrderUserInfo.setAddress(cmMallAddress.getAddressDetail());
                bpUserInfoMapper.insertUserInfo(cmMallBpOrderUserInfo);
            } else {
                return cmWxJsonModel.error("订单地址异常");
            }


            //保存发票信息(发票采用不可选，只要用户存在增值税发票则保存最新记录即可)
            CmMallUserInvoiceInfo invoiceInfo = personalMapper.findInvoice(userId);
            if (null != invoiceInfo) {
                CmMallOrderInvoice cmMallOrderInvoice = new CmMallOrderInvoice();
                cmMallOrderInvoice.setOrderId(orderID.intValue());
                cmMallOrderInvoice.setInvoiceTitle(invoiceInfo.getInvoiceTitle());
                cmMallOrderInvoice.setType("2");//增值税发票
                cmMallOrderInvoice.setCorporationTaxNum(invoiceInfo.getCorporationTaxNum());
                cmMallOrderInvoice.setRegisteredAddress(invoiceInfo.getRegisteredAddress());
                cmMallOrderInvoice.setRegisteredPhone(invoiceInfo.getRegisteredPhone());
                cmMallOrderInvoice.setBankAccountNo(invoiceInfo.getBankAccountNo());
                cmMallOrderInvoice.setOpenBank(invoiceInfo.getOpenBank());
                orderMapper.insertOrderInvoice(cmMallOrderInvoice);
            }

            //提交订单完成后清理购物车(用户ID,商品ID)
            if (StringUtils.equals("1", cartType)) {//购物车提交后清理购物车
                for (CmMallOrderProduct product : orderProductList) {//循环插入数据
                    shoppingMapper.deleteCart(userId, product.getOrganizeProductID());
                }
            }
        }

        /**构造返回参数*/
        Map<String, String> info = new HashMap<>();
        info.put("orderID", String.valueOf(cmMallOrder.getOrderID()));
        info.put("orderNo", String.valueOf(cmMallOrder.getOrderNo()));
        info.put("orderMark", "#" + String.valueOf(cmMallOrder.getOrderID()) + "#");
        info.put("payTotalFee", String.valueOf(cmMallOrder.getPayTotalFee()));//应付订单金额
        info.put("payableAmount", String.valueOf(cmMallOrder.getPayableAmount()));//真实需要付款金额
        if (isPaySuccessFlag) {//余额抵扣成功
            return cmWxJsonModel.success("1", info, "提交成功且已支付");//1提交成功[且支付完成]
        } else {
            return cmWxJsonModel.success("2", info, "提交成功但未支付");
        }
    }

    /**
     * 获取阶梯价格最小购买数量
     * @return
     */
    private Integer getMinBuyNumber(List<CmMallProductLadderPrice> ladderPriceList) {
        Integer minBuyNumber = 1;
        //阶梯价格信息
        if (!CollectionUtils.isEmpty(ladderPriceList)) {
            CmMallProductLadderPrice productLadderPrice0 = ladderPriceList.get(0);//第一阶梯
            if (null != productLadderPrice0) minBuyNumber = productLadderPrice0.getBuyNum();
        }
        return minBuyNumber;
    }

    @Override
    public List<CmMallLogisticsBatch> logistics(Integer orderID) throws Exception {
        //物流商品信息
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) throw new Exception("订单已删除");
        List<CmMallLogisticsBatch> batchList = orderMapper.findLogistics(orderID);
        for (CmMallLogisticsBatch batch : batchList) {
            List<String> shopOrderIDList = orderMapper.findShopOrderID(orderID, batch.getId());
            String[] ShopOrderIDs = shopOrderIDList.toArray(new String[shopOrderIDList.size()]);
            List<CmMallShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
            for (CmMallShopOrder shopOrder : shopOrderList) {
                List<CmMallLogisticsRecord> logisticsRecordList = orderMapper.findLogisticsRecord(shopOrder.getShopOrderID(), batch.getId());
                for (CmMallLogisticsRecord logisticsRecord : logisticsRecordList) {
                    logisticsRecord.setImage(AppUtils.getImageURL("product", logisticsRecord.getImage(), 0, domain));
                }
                shopOrder.setCmLogisticsRecords(logisticsRecordList);
                shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
            }
            //物流信息
            List<CmMallLogisticsInfo> cmMallLogisticsInfoList = orderMapper.findLogisticsInfo(batch.getId());
            for (CmMallLogisticsInfo cmMallLogisticsInfo : cmMallLogisticsInfoList) {
                List<CmMallRouter> cmMallRouters = JSONArray.parseArray(cmMallLogisticsInfo.getInfo(), CmMallRouter.class);
                cmMallLogisticsInfo.setRouters(cmMallRouters);
            }
            batch.setLogisticsInfos(cmMallLogisticsInfoList);
            batch.setShopOrderList(shopOrderList);
        }
        return batchList;
    }

    @Override
    public List<CmMallOrder> myOrder(Integer userID, Integer orderState, Integer organizeID) {
        List<CmMallOrder> orderList = orderMapper.findAllOrder(userID, orderState, organizeID);
        for (CmMallOrder order : orderList) {
            //111,待付待收待发
            if ("11".equals(order.getStatus()) && "1".equals(order.getPayStatus())) {
                order.setStatus("111");
            }
            String[] ShopOrderIDs = {};
            if (order.getShopOrderIDs().contains(",")) {
                ShopOrderIDs = order.getShopOrderIDs().split(",");
            } else {
                ShopOrderIDs = new String[]{order.getShopOrderIDs()};
            }
            List<CmMallShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
            for (CmMallShopOrder shopOrder : shopOrderList) {
                List<CmMallOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
                for (CmMallOrderProduct orderProduct : orderProductList) {
                    orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
                }
                shopOrder.setCmOrderProducts(orderProductList);
                shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
            }
            order.setShopOrderList(shopOrderList);
        }
        return orderList;
    }

    @Override
    public Map<String, Object> oftenPurchase(Integer userID, Integer organizeID, Integer index, Integer pageSize) {
        Map<String, Object> map = new HashMap();
        if (index == null) index = 1;
        if (pageSize == null) pageSize = 10;
        PageHelper.startPage(index, pageSize);
        //商品分页数据
        List<CmMallOrganizeProducts> organizeProductsList = orderMapper.oftenPurchase(userID, organizeID);
        for (CmMallOrganizeProducts organizeProducts : organizeProductsList) {
            organizeProducts.setMainImage(AppUtils.getImageURL("product", organizeProducts.getMainImage(), 0, domain));
            if (organizeProducts.getLadderPriceFlag().equals("1")) {
                //启用阶梯价格
                CmMallProductLadderPrice ladderPrice = orderMapper.findLadderPrice(organizeProducts.getId());
                organizeProducts.setRetailPrice(ladderPrice.getBuyPrice());
                organizeProducts.setMinBuyNumber(ladderPrice.getBuyNum());
            }
        }
        CmMallPage<CmMallOrganizeProducts> page = new CmMallPage<>(organizeProductsList);
        //购物车数量
        Integer count = homePageMapper.cartQuantity(userID);
        if (count == null) count = 0;
        map.put("page", page);
        map.put("count", count);
        return map;
    }

    @Override
    public CmWxJsonModel deleteOrder(Integer orderID) {
        CmWxJsonModel model = CmWxJsonModel.newInstance();
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) return model.error("订单不存在");
        if (!("6".equals(order.getStatus()))) {
            return model.error("订单异常");
        }
        //逻辑删除主订单
        order.setDelFlag("1");
        order.setNote("用户发起删除订单");
        orderMapper.update(order);
        //逻辑删除子订单
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        for (String shopOrderID : ShopOrderIDs) {
            CmMallShopOrder shopOrder = cmMallShopOrderMapper.get(Integer.valueOf(shopOrderID));
            shopOrder.setDelFlag("1");
            cmMallShopOrderMapper.update(shopOrder);
        }

        //逻辑删除收款记录
        List<CmMallDiscernReceipt> discernReceipt = orderMapper.findDiscernReceipt(orderID);
        if (discernReceipt != null && discernReceipt.size() > 0) {
            orderMapper.deleteReceiptOrderRelation(orderID);
            for (CmMallDiscernReceipt receipt : discernReceipt) {
                orderMapper.deleteDiscernReceipt(receipt.getId());
            }
        }
        //逻辑删除付款记录
        for (String shopOrderID : ShopOrderIDs) {
            List<CmMallPayShopRecord> payShopRecordList = orderMapper.findPayShopRecord(Integer.valueOf(shopOrderID));
            if (payShopRecordList != null && payShopRecordList.size() > 0) {
                orderMapper.deletePayShopRecord(Integer.valueOf(shopOrderID));
                for (CmMallPayShopRecord payShopRecord : payShopRecordList) {
                    orderMapper.updatePayShop(payShopRecord.getPayShopID());
                }
            }
        }
        return model.success("删除订单成功", "");
    }

    @Override
    public void cancelOrder(Integer orderID) throws Exception {
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) { throw new Exception("订单不存在"); }
        if (!("11".equals(order.getStatus()))) {
            throw new Exception("订单异常");
        }
        order.setStatus("6");
        orderMapper.update(order);
    }

    @Override
    public void affirmCargo(Integer orderID) throws Exception {
        CmMallOrder order = orderMapper.findOrder(orderID);
        if (order == null) { throw new Exception("订单不存在"); }
        if ("33".equals(order.getStatus()) && "3".equals(order.getPayStatus())) {
            //已付款供应商,订单完成
            order.setStatus("5");
            orderMapper.update(order);
            orderMapper.updateLogisticsBatch(orderID);
        } else if ("33".equals(order.getStatus())) {
            //交易完成
            order.setStatus("4");
            orderMapper.update(order);
            orderMapper.updateLogisticsBatch(orderID);
        } else {
            throw new Exception("订单异常");
        }
    }
}
