package com.caimei.service.products.impl;

import com.caimei.entity.*;
import com.caimei.mapper.products.DetailsMapper;
import com.caimei.mapper.products.HomePageMapper;
import com.caimei.service.products.DetailsService;
import com.caimei.utils.AppUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class DetailsServiceImpl implements DetailsService {
    @Autowired
    private DetailsMapper detailsMapper;
    @Autowired
    private HomePageMapper homePageMapper;
    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public CmMallOrganizeProducts productDetails(Integer id, Integer userID) {
        //商品信息
        CmMallOrganizeProducts organizeProducts = detailsMapper.findProduct(id);
        CmMallProduct cmMallProduct = detailsMapper.product(organizeProducts.getProductID());
        String[] split = null;
        String tags = "";
        //商品标签
        if (cmMallProduct.getTags() != null) {
            tags = cmMallProduct.getTags();
            tags = tags.replace(" ", ",");
            tags = tags.replace("、", ",");
            tags = tags.replace("，", ",");
            if (tags.contains(",")) {
                split = tags.split(",");
                List<String> list = new ArrayList();
                for (String s : split) {
                    if (s != null && !s.equals("")) list.add(s);
                    split = list.toArray(new String[list.size()]);
                }
            }
        }
        cmMallProduct.setTagsList(split);
        cmMallProduct.setMainImage(AppUtils.getImageURL("product", cmMallProduct.getMainImage(), 0, domain));
        if (cmMallProduct.getStock() == null || cmMallProduct.getStock() == 0) cmMallProduct.setStock(99);
        organizeProducts.setProduct(cmMallProduct);
        //商品图片
        List<CmMallProductImage> cmMallProductImage = detailsMapper.findAllProductImage(organizeProducts.getProductID());
        for (CmMallProductImage image : cmMallProductImage) {
            String imageURL = AppUtils.getImageURL("product", image.getImage(), 0, domain);
            image.setImage(imageURL);
        }
        organizeProducts.setProductImage(cmMallProductImage);
        //商品详情
        CmMallProductDetailInfo cmMallProductDetailInfo = detailsMapper.findProductDetail(organizeProducts.getProductID());
        organizeProducts.setProductDetailInfo(cmMallProductDetailInfo);
        //供应商信息
        CmMallShop cmMallShop = detailsMapper.findShop(cmMallProduct.getShopID());
        if (cmMallShop.getBusinessScope() != null && cmMallShop.getBusinessScope().contains("/")) {
            String[] businessScopeArray = cmMallShop.getBusinessScope().split("/");
            cmMallShop.setBusinessScopeArray(businessScopeArray);
        }
        cmMallShop.setLogo(AppUtils.getImageURL("shopLogo", cmMallShop.getLogo(), 0, domain));
        cmMallShop.setBusinessLicenseImage(AppUtils.getImageURL("shopLogo", cmMallShop.getBusinessLicenseImage(), 0, domain));
        organizeProducts.setShop(cmMallShop);
        //购物车数量
        Integer count = homePageMapper.cartQuantity(userID);
        if (count == null) count = 0;
        organizeProducts.setCartCount(count);
        //启用阶梯价格
        if (organizeProducts.getLadderPriceFlag().equals("1")) {
            List<CmMallProductLadderPrice> ladderPriceList = detailsMapper.findLadderPrice(id);
            //已根据购买期数排序，期数越大价格越低
            Integer buyNum1 = 0;//第一阶梯（价格最高）
            Integer buyNum2 = 0;//第二阶梯
            Integer buyNum3 = 0;//第三阶梯（价格最低）
            if (ladderPriceList.size() == 3) {
                CmMallProductLadderPrice ladderPrice0 = ladderPriceList.get(0);//第一阶梯
                CmMallProductLadderPrice ladderPrice1 = ladderPriceList.get(1);//第二阶梯
                CmMallProductLadderPrice ladderPrice2 = ladderPriceList.get(2);//第三阶梯
                if (null != ladderPrice0) buyNum1 = ladderPrice0.getBuyNum();
                if (null != ladderPrice1) buyNum2 = ladderPrice1.getBuyNum();
                if (null != ladderPrice2) buyNum3 = ladderPrice2.getBuyNum();
                //增加阶梯价格区间属性设置
                ladderPrice0.setBuyNumRangeShow(buyNum1 + "~" + (buyNum2 - 1));
                ladderPrice1.setBuyNumRangeShow(buyNum2 + "~" + (buyNum3 - 1));
                ladderPrice2.setBuyNumRangeShow("≥" + buyNum3);
            } else if (ladderPriceList.size() == 2) {
                CmMallProductLadderPrice ladderPrice0 = ladderPriceList.get(0);//第一阶梯
                CmMallProductLadderPrice ladderPrice1 = ladderPriceList.get(1);//第二阶梯
                if (null != ladderPrice0) buyNum1 = ladderPrice0.getBuyNum();
                if (null != ladderPrice1) buyNum2 = ladderPrice1.getBuyNum();
                //增加阶梯价格区间属性设置
                ladderPrice0.setBuyNumRangeShow(buyNum1 + "~" + (buyNum2 - 1));
                ladderPrice1.setBuyNumRangeShow("≥" + buyNum2);
            }
            organizeProducts.setRetailPrice(ladderPriceList.get(0).getBuyPrice());
            organizeProducts.setMinBuyNumber(ladderPriceList.get(0).getBuyNum());
            organizeProducts.setProductLadderPrices(ladderPriceList);
        }
        return organizeProducts;
    }

    @Override
    public Integer saveCart(CmMallCart cmMallCart) {
        CmMallCart cart = detailsMapper.findCartProduct(cmMallCart);
        if (cart != null) {
            int count = cart.getProductCount() + cmMallCart.getProductCount();
            cart.setProductCount(count);
            cart.setAddTime(new Date());
            detailsMapper.updateCart(cart);
        } else {
            cmMallCart.setAddTime(new Date());
            cmMallCart.setDelFlag("0");
            detailsMapper.saveCart(cmMallCart);
        }
        Integer cartQuantity = homePageMapper.cartQuantity(cmMallCart.getUserID());
        if (cartQuantity == null) cartQuantity = 0;
        return cartQuantity;
    }
}
