package com.caimei.service.products.impl;

import com.caimei.entity.*;
import com.caimei.mapper.products.HomePageMapper;
import com.caimei.entity.CmMallOrganizeProducts;
import com.caimei.module.search.entity.vo.CmMallProductLadderPriceVo;
import com.caimei.module.search.service.SearchService;
import com.caimei.service.products.HomePageService;
import com.caimei.utils.AppUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class HomePageServiceImpl implements HomePageService {
    protected static final Logger logger = LoggerFactory.getLogger(HomePageServiceImpl.class);
    @Autowired
    private HomePageMapper homePageMapper;
    @Autowired
    private SearchService searchService;
    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public Map<String, Object> information(Integer organizeID, Integer userID) {
        logger.info("------------HomePageServiceImpl.information.organizeID:" + organizeID + ",userID:" + userID);
        Map<String, Object> map = new HashMap();
        //商品分类
        List<CmMallProductsClassify> productsClassify = homePageMapper.findClassify(organizeID);
        //常用商品
        List<CmMallOrganizeProducts> organizeProducts = homePageMapper.findProducts(organizeID);
        for (CmMallOrganizeProducts product : organizeProducts) {
            product.setMainImage(AppUtils.getImageURL("product", product.getMainImage(), 0, domain));
            //是否启用阶梯价格
            if (product.getLadderPriceFlag().equals("1")) {
                CmMallProductLadderPriceVo ladderPrice = searchService.findLadderPrice(product.getId().longValue());
                product.setRetailPrice(ladderPrice.getBuyPrice().doubleValue());
            }
        }
        //购物车数量
        Integer count = homePageMapper.cartQuantity(userID);
        if (count == null) count = 0;
        logger.info("------------HomePageServiceImpl.information.count:" + count);
        //模块数据
        CmMallPageModules mallPageModules = homePageMapper.findModules(organizeID);
        //banner图
        List<String> bannerImageList = homePageMapper.findBannerImage(organizeID);
        map.put("productsClassify", productsClassify);
        map.put("organizeProducts", organizeProducts);
        map.put("count", count);
        map.put("mallPageModules", mallPageModules);
        map.put("bannerImageList", bannerImageList);
        return map;
    }

    @Override
    public List<CmMallOrganizeProducts> classify(CmMallOrganizeProducts organizeProducts) {
        List<CmMallOrganizeProducts> classify = homePageMapper.classify(organizeProducts);
        for (CmMallOrganizeProducts products : classify) {
            products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
            //是否启用阶梯价格
            if (products.getLadderPriceFlag().equals("1")) {
                CmMallProductLadderPriceVo ladderPrice = searchService.findLadderPrice(products.getId().longValue());
                products.setRetailPrice(ladderPrice.getBuyPrice().doubleValue());
            }
        }
        return classify;
    }

    @Override
    public List<CmMallOrganizeProducts> preferred(CmMallOrganizeProducts organizeProducts) {
        List<CmMallOrganizeProducts> preferred = homePageMapper.preferred(organizeProducts);
        for (CmMallOrganizeProducts products : preferred) {
            products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
            //是否启用阶梯价格
            if (products.getLadderPriceFlag().equals("1")) {
                CmMallProductLadderPriceVo ladderPrice = searchService.findLadderPrice(products.getId().longValue());
                products.setRetailPrice(ladderPrice.getBuyPrice().doubleValue());
            }
        }
        return preferred;
    }

    @Override
    public List<CmMallHospital> member() {
        List<CmMallHospital> member = homePageMapper.member();
        for (CmMallHospital CmMallHospital : member) {
            List<CmMallBrand> CmMallBrands = homePageMapper.findId(CmMallHospital.getId());
            CmMallHospital.setBrands(CmMallBrands);
        }
        return member;
    }

    @Override
    public CmMallOrganize afterSale(Integer organizeID) {
        return homePageMapper.afterSale(organizeID);
    }
}
